@@ -26,6 +26,8 @@ module Senna
   SYM_WITH_SIS                  =  0x80000000
 
   SNIP_NORMALIZE                =      0x0001
+  SNIP_COPY_TAG                 =      0x0002
+  SNIP_SKIP_LEADING_SPACES      =      0x0004
   SNIP_MAP_HTMLENCODE           =          -1
 
   RC_SUCCESS                    =      0x0000
@@ -57,8 +59,11 @@ module Senna
   SEL_PARTIAL                   =      0x0001
   SEL_UNSPLIT                   =      0x0002
   SEL_NEAR                      =      0x0003
-  SEL_SIMILAR                   =      0x0004
-  SEL_TERM_EXTRACT              =      0x0005
+  SEL_NEAR2                     =      0x0004
+  SEL_SIMILAR                   =      0x0005
+  SEL_TERM_EXTRACT              =      0x0006
+  SEL_PREFIX                    =      0x0007
+  SEL_SUFFIX                    =      0x0008
 
   SORT_DESCENDING               =      0x0000
   SORT_ASCENDING                =      0x0001
@@ -87,6 +92,10 @@ module Senna
   CTX_QUIET                     =      0x0008
   CTX_QUIT                      =      0x0010
 
+  STR_REMOVEBLANK               =      0x0001
+  STR_WITH_CTYPES               =      0x0002
+  STR_WITH_CHECKS               =      0x0004
+
   class Index
     attr_reader :handle
     attr_reader :key_size, :flags, :initial_n_segments, :encoding
@@ -184,9 +193,10 @@ module Senna
 
     def rename(newpath)
       _path = path
-      Senna::sen_index_close(@handle)
+      close
       Senna::sen_index_rename(_path, newpath)
       @handle = Senna::sen_index_open(newpath)
+      ObjectSpace.define_finalizer(self, Index.callback(@handle))
     end
 
     def inv_check(term)
@@ -715,6 +725,23 @@ module Senna
       Senna::sen_query_exec(index.handle, @handle, records.handle, op)
       return records
     end
+
+    def term
+      # TODO: implement!
+      raise NameError, 'Query#term is not implemented'
+    end
+
+    def scan(strs, flags)
+      # TODO: implement!
+      # [found, score]
+      raise NameError, 'Query#scan is not implemented'
+    end
+
+    def snip(flags, width, max_results, tags)
+      # TODO: implement!
+      # Snip.new(handle)
+      raise NameError, 'Query#snip is not implemented'
+    end
   end
 
   class Snip
@@ -896,4 +923,8 @@ module Senna
     sen_logger_put(level, '', 0, '', '%s', msg)
   end
 
+  def Senna::str_normalize(str, encoding, flags)
+    l, = sen_str_normalize(str, str.size, encoding, flags, 0)
+    return sen_str_normalize(str, str.size, encoding, flags, l + 1)[1]
+  end
 end
