@@ -7,16 +7,18 @@
 
 require 'test/unit'
 require 'senna'
-require 'nkf'
 require 'test_tools'
 
 $KCODE = 'e'
 
 class IndexUpdateTest < Test::Unit::TestCase
+  include TestTools
+
   SOME = 1000
   MANY = 20000
   MAX_KEY_LEN = 8191
   MAX_KEY_NUM = 0x1000_0000 - 1
+
   def test_upd_sel
     index = TestIndex::create
     index.upd('key', nil, 'test')
@@ -86,28 +88,28 @@ class IndexUpdateTest < Test::Unit::TestCase
 
   def test_normalize_sjis_upd
     index = TestIndex::create(0, Senna::INDEX_NORMALIZE | Senna::INDEX_NGRAM, 0, Senna::ENC_SJIS)
-    index.upd('', nil, NKF.nkf('-Es', 'ԣӣ'))
-    rcs = index.sel(NKF.nkf('-Es', 'ţ'))
+    index.upd('', nil, euc2sjis('ԣӣ'))
+    rcs = index.sel(euc2sjis('ţ'))
     assert_equal(1, rcs.nhits)
-    index.upd('', NKF.nkf('-Es', 'ӣ'), NKF.nkf('-Esx', 'ݎ'))
-    rcs = index.sel(NKF.nkf('-Es', 'ţ'))
+    index.upd('', euc2sjis('ӣ'), euc2sjis('ݎ'))
+    rcs = index.sel(euc2sjis('ţ'))
     assert_nil(rcs)
-    rcs = index.sel(NKF.nkf('-Es', ''))
+    rcs = index.sel(euc2sjis(''))
     assert_equal(1, rcs.nhits)
   end
 
   def test_normalize_utf8_upd
     index = TestIndex::create(0, Senna::INDEX_NORMALIZE | Senna::INDEX_NGRAM, 0, Senna::ENC_UTF8)
-    index.upd('', nil, NKF.nkf('-Ew', 'ԣӣ'))
-    rcs = index.sel(NKF.nkf('-Ew', 'ţ'))
+    index.upd('', nil, euc2utf8('ԣӣ'))
+    rcs = index.sel(euc2utf8('ţ'))
     assert_equal(1, rcs.nhits)
-    index.upd('', NKF.nkf('-Ew', 'ӣ'), NKF.nkf('-Ewx', 'ݎ'))
-    index.upd('2', nil, NKF.nkf('-Ew', ''))
-    index.upd('3', nil, NKF.nkf('-Ew', '󥷡'))
-    rcs = index.sel(NKF.nkf('-Ew', 'ţ'))
+    index.upd('', euc2utf8('ӣ'), euc2utf8('ݎ'))
+    index.upd('2', nil, euc2utf8(''))
+    index.upd('3', nil, euc2utf8('󥷡'))
+    rcs = index.sel(euc2utf8('ţ'))
     assert_nil(rcs)
     ['', 'ߥС', ''].each {|word|
-      rcs = index.sel(NKF.nkf('-Ew', word))
+      rcs = index.sel(euc2utf8(word))
       assert_equal(1, rcs.nhits)
     }
   end
@@ -136,7 +138,7 @@ class IndexUpdateTest < Test::Unit::TestCase
     assert_not_equal(0, rcs.find(1))
     assert_not_equal(0, rcs.find(MAX_KEY_NUM))
   end
-  
+
   def test_max_key_upd
     index = TestIndex::create
     10.times {|i|
@@ -155,20 +157,21 @@ class IndexUpdateTest < Test::Unit::TestCase
   end
 
   def test_ngram_eng
-    index = TestIndex::create(0, Senna::INDEX_NGRAM)
+    index = TestIndex::create(0, Senna::INDEX_NGRAM | Senna::INDEX_NORMALIZE)
     index.upd('key', nil, 'test')
     index.upd('key2', nil, 'te')
     index.upd('key3', nil, 's')
     rcs = index.sel('test')
     assert_equal(1, rcs.nhits)
     rcs = index.sel('te')
-    assert_equal(2, rcs.nhits)
+    assert_equal(1, rcs.nhits)
     rcs = index.sel('s')
-    assert_equal(1, rcs.nhits) 
+    assert_equal(1, rcs.nhits)
   end
 
   def test_ngram_ja
-    index = TestIndex::create(0, Senna::INDEX_NGRAM, 0, Senna::ENC_EUC_JP)
+    index = TestIndex::create(0, Senna::INDEX_NGRAM | Senna::INDEX_NORMALIZE,
+                              0, Senna::ENC_EUC_JP)
     index.upd('key', nil, 'ƥ')
     index.upd('key2', nil, 'ƥ')
     index.upd('key3', nil, '')
@@ -177,7 +180,7 @@ class IndexUpdateTest < Test::Unit::TestCase
     rcs = index.sel('ƥ')
     assert_equal(2, rcs.nhits)
     rcs = index.sel('')
-    assert_equal(1, rcs.nhits)
+    assert_equal(3, rcs.nhits)
   end
 
   def test_ngram_normalize
