@@ -15,16 +15,16 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifndef _SEN_COM_H
-#define _SEN_COM_H
+#ifndef SEN_COM_H
+#define SEN_COM_H
 
-#ifndef _SENNA_H
+#ifndef SENNA_H
 #include "senna_in.h"
-#endif /* _SENNA_H */
+#endif /* SENNA_H */
 
-#ifndef _SEN_STR_H
+#ifndef SEN_STR_H
 #include "str.h"
-#endif /* _SEN_STR_H */
+#endif /* SEN_STR_H */
 
 #ifdef	__cplusplus
 extern "C" {
@@ -36,11 +36,17 @@ extern "C" {
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
+#define SEN_COM_POLLIN  1
+#define SEN_COM_POLLOUT 2
 #else /* USE_SELECT */
 #ifdef USE_EPOLL
 #include <sys/epoll.h>
+#define SEN_COM_POLLIN  EPOLLIN
+#define SEN_COM_POLLOUT EPOLLOUT
 #else /* USE_EPOLL */
 #include <sys/poll.h>
+#define SEN_COM_POLLIN  POLLIN
+#define SEN_COM_POLLOUT POLLOUT
 #endif /* USE_EPOLL */
 #endif /* USE_SELECT */
 
@@ -49,11 +55,14 @@ typedef struct _sen_com_event sen_com_event;
 typedef void sen_com_callback(sen_com_event *, sen_com *);
 
 enum {
-  sen_com_start = 0,
+  sen_com_idle = 0,
   sen_com_head,
   sen_com_body,
+  sen_com_doing,
   sen_com_done,
-  sen_com_error
+  sen_com_connecting,
+  sen_com_error,
+  sen_com_closing
 };
 
 enum {
@@ -65,10 +74,9 @@ enum {
 };
 
 struct _sen_com {
-  int fd;
-  int events; // todo : support POLLOUT
-  uint16_t stat_in;
-  uint16_t stat_out;
+  sen_sock fd;
+  uint8_t status;
+  int events;
   sen_com_callback *ev_in;
   sen_com_callback *ev_out;
 };
@@ -82,6 +90,7 @@ struct _sen_com_event {
   int epfd;
   struct epoll_event *events;
 #else /* USE_EPOLL */
+  int dummy; /* dummy */
   struct pollfd *events;
 #endif /* USE_EPOLL */
 #endif /* USE_SELECT */
@@ -91,8 +100,9 @@ sen_rc sen_com_init(void);
 void sen_com_fin(void);
 sen_rc sen_com_event_init(sen_com_event *ev, int max_nevents, int data_size);
 sen_rc sen_com_event_fin(sen_com_event *ev);
-sen_rc sen_com_event_add(sen_com_event *ev, int fd, sen_com **com);
-sen_rc sen_com_event_del(sen_com_event *ev, int fd);
+sen_rc sen_com_event_add(sen_com_event *ev, sen_sock fd, int events, sen_com **com);
+sen_rc sen_com_event_mod(sen_com_event *ev, sen_sock fd, int events, sen_com **com);
+sen_rc sen_com_event_del(sen_com_event *ev, sen_sock fd);
 sen_rc sen_com_event_poll(sen_com_event *ev, int timeout);
 sen_rc sen_com_event_each(sen_com_event *ev, sen_com_callback *func);
 
@@ -100,11 +110,6 @@ sen_rc sen_com_event_each(sen_com_event *ev, sen_com_callback *func);
 
 #define SEN_COM_PROTO_SQTP 20819
 
-#define SEN_COM_SQTP_HEAD  1
-#define SEN_COM_SQTP_TAIL  2
-#define SEN_COM_SQTP_MORE  4
-#define SEN_COM_SQTP_QUIET 8
-
 typedef struct _sen_com_sqtp sen_com_sqtp;
 typedef struct _sen_com_sqtp_header sen_com_sqtp_header;
 
@@ -142,4 +147,4 @@ sen_rc sen_com_sqtp_recv(sen_com_sqtp *cs, sen_rbuf *buf,
 }
 #endif
 
-#endif /* _SEN_COM_H */
+#endif /* SEN_COM_H */
