@@ -570,6 +570,7 @@ sen_index_select(sen_index *i, const char *string, unsigned int string_len,
                  sen_records *r, sen_sel_operator op, sen_select_optarg *optarg)
 {
   sen_rc rc;
+  if (!r || !i) { return sen_invalid_argument; }
   r->keys = i->keys;
   if (optarg) { optarg->max_size = sen_sym_size(i->keys) * sizeof(int); }
   rc = sen_inv_select(i->inv, string, string_len, r, op, optarg);
@@ -580,11 +581,21 @@ sen_index_select(sen_index *i, const char *string, unsigned int string_len,
 sen_records *
 sen_index_sel(sen_index *i, const char *string, unsigned int string_len)
 {
-  sen_records *r = sen_inv_sel(i->inv, string, string_len);
+  sen_records *r;
+  if (!i) { return NULL; }
+  r = sen_inv_sel(i->inv, string, string_len);
   if (r) { r->keys = i->keys; }
   return r;
 }
 
+#ifdef USE_QUERY_ABORT
+void
+sen_index_set_abort_callback(sen_index *i, int (*cb)(void*), void *arg)
+{
+  sen_inv_set_abort_callback(i->inv, cb, arg);
+}
+#endif /* USE_QUERY_ABORT */
+
 /* sen_records_heap class */
 
 struct _sen_records_heap {
