@@ -51,6 +51,10 @@ struct _sen_inv {
   sen_io *chunk;
   sen_sym *lexicon;
   struct sen_inv_header *header;
+#ifdef USE_QUERY_ABORT
+  int (*check_abort)(void*);
+  void *check_abort_arg;
+#endif /* USE_QUERY_ABORT */
 };
 
 struct sen_inv_header {
@@ -1302,6 +1306,10 @@ sen_inv_create(const char *path, sen_sym *lexicon, uint32_t initial_n_segments)
   inv->chunk = chunk;
   inv->header = header;
   inv->lexicon = lexicon;
+#ifdef USE_QUERY_ABORT
+  inv->check_abort = NULL;
+  inv->check_abort_arg = NULL;
+#endif /* USE_QUERY_ABORT */
   inv->header->total_chunk_size = 0;
   SEN_GET(path,inv);
   return inv;
@@ -1408,6 +1416,15 @@ sen_inv_expire(void)
   }
 }
 
+#ifdef USE_QUERY_ABORT
+void
+sen_inv_set_abort_callback(sen_inv *inv, int (*cb)(void*), void *arg)
+{
+  inv->check_abort = cb;
+  inv->check_abort_arg = arg;
+}
+#endif /* USE_QUERY_ABORT */
+
 sen_rc
 sen_inv_update_one(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int hint)
 {
@@ -2249,12 +2266,19 @@ token_info_expand_both(sen_sym *lexicon, sen_inv *inv, const char *key, token_in
   uint32_t *offset2;
   sen_set_cursor *c;
   sen_id *tp, *tq;
+#ifdef USE_QUERY_ABORT
+  int check_abort_cnt = 0;
+#endif /* USE_QUERY_ABORT */
   if ((h = sen_sym_prefix_search(lexicon, key))) {
     // sen_log("key=%s h->n=%d", key, h->n_entries);
     if ((ti->cursors = cursor_heap_open(h->n_entries + 256))) {
       if ((c = sen_set_cursor_open(h))) {
         while (sen_set_cursor_next(c, (void **) &tp, NULL)) {
           const char *key2 = _sen_sym_key(lexicon, *tp);
+#ifdef USE_QUERY_ABORT
+	  if (inv->check_abort != NULL && (++check_abort_cnt & 0x3f) == 0
+	      && (*inv->check_abort)(inv->check_abort_arg)) { break; }
+#endif /* USE_QUERY_ABORT */
           if (!key2) { break; }
           // sen_log("key2=%s", key2);
           if (sen_str_len(key2, lexicon->encoding, NULL) == 1) { // todo:
@@ -3202,6 +3226,7 @@ sen_inv_select(sen_inv *inv, const char *string, unsigned int string_len,
       sen_inv_posting *p = c->post;
       if ((weight = get_weight(s, p->rid, p->sid, wvm, optarg))) {
         SEN_SET_INT_ADD(s, p, ri);
+        if (!ri) { goto exit; }
         ri->score = (p->tf + p->score) * weight;
         ri->n_subrecs = 1;
       }
@@ -3286,6 +3311,11 @@ sen_inv_select(sen_inv *inv, const char *string, unsigned int string_len,
       }
     }
     if (token_info_skip(*tis, nrid, nsid)) { goto exit; }
+#ifdef USE_QUERY_ABORT
+    if (inv->check_abort != NULL && (*inv->check_abort)(inv->check_abort_arg)) {
+      goto exit;
+    }
+#endif /* USE_QUERY_ABORT */
   }
 exit :
   for (tip = tis; tip < tis + n; tip++) {
@@ -3334,7 +3364,10 @@ sen_inv_sel(sen_inv *inv, const char *string, unsigned int string_len)
     sen_select_optarg arg = {sen_sel_exact, 0, 0, NULL, 0, NULL, NULL};
     sen_set *s = SEN_MALLOC(sizeof(sen_set));
     if (!s) { return NULL; }
-    sen_rset_init(ctx, s, sen_rec_document, 0, sen_rec_none, 0, 0);
+    if (sen_rset_init(ctx, s, sen_rec_document, 0, sen_rec_none, 0, 0)) {
+      SEN_FREE(s);
+      return NULL;
+    }
     if (sen_inv_select(inv, string, string_len, s, sen_sel_or, &arg)) {
       SEN_LOG(sen_log_error, "sen_inv_select on sen_inv_sel(1) failed !");
       sen_set_close(s);
