@@ -80,6 +80,10 @@ uint32_t sen_inv_estimate_size(sen_inv *inv, uint32_t key);
 void sen_inv_seg_expire(sen_inv *inv, int32_t threshold);
 void sen_inv_expire(void);
 
+#ifdef USE_QUERY_ABORT
+void sen_inv_set_abort_callback(sen_inv *inv, int (*cb)(void*), void *arg);
+#endif /* USE_QUERY_ABORT */
+
 typedef struct {
   sen_id rid;
   uint32_t sid;
