@@ -225,6 +225,7 @@ sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
 {
   unsigned int bufsize, maxtrial = 10, len;
   char *buf, *s, *p;
+  char mecab_err[256];
   sen_lex *lex;
   sen_ctx *ctx = nstr->ctx;
   if (!(lex = SEN_MALLOC(sizeof(sen_lex)))) { return NULL; }
@@ -248,6 +249,7 @@ sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
   lex->encoding = sym->encoding;
   lex->nstr = nstr;
   len = nstr->norm_blen;
+  mecab_err[sizeof(mecab_err) - 1] = '\0';
   for (bufsize = len * 2 + 1; maxtrial; bufsize *= 2, maxtrial--) {
     if(!(buf = SEN_MALLOC(bufsize + 1))) {
       SEN_LOG(sen_log_alert, "buffer allocation on sen_mecab_open failed !");
@@ -256,12 +258,18 @@ sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
     }
     MUTEX_LOCK(sole_mecab_lock);
     s = mecab_sparse_tostr3(lex->mecab, (char *)nstr->norm, len, buf, bufsize);
+    if (!s) {
+      strncpy(mecab_err, mecab_strerror(lex->mecab), sizeof(mecab_err) - 1);
+    }
     MUTEX_UNLOCK(sole_mecab_lock);
     if (s) { break; }
     SEN_FREE(buf);
+    if (strstr(mecab_err, "output buffer overflow") == NULL) {
+      break;
+    }
   }
-  if (!maxtrial) {
-    SEN_LOG(sen_log_alert, "mecab_sparse_tostr failed len=%d bufsize=%d", len, bufsize);
+  if (!s) {
+    SEN_LOG(sen_log_alert, "mecab_sparse_tostr failed len=%d bufsize=%d err=%s", len, bufsize, mecab_err);
     sen_lex_close(lex);
     return NULL;
   }
