@@ -18,20 +18,20 @@
 #include <mecab.h>
 #endif /* NO_MECAB */
 
-#ifndef _SEN_LEX_H
-#define _SEN_LEX_H
+#ifndef SEN_LEX_H
+#define SEN_LEX_H
 
-#ifndef _SENNA_H
+#ifndef SENNA_H
 #include "senna_in.h"
-#endif /* _SENNA_H */
+#endif /* SENNA_H */
 
-#ifndef _SEN_SYM_H
+#ifndef SEN_SYM_H
 #include "sym.h"
-#endif /* _SEN_SYM_H */
+#endif /* SEN_SYM_H */
 
-#ifndef _SEN_STR_H
+#ifndef SEN_STR_H
 #include "str.h"
-#endif /* _SEN_STR_H */
+#endif /* SEN_STR_H */
 
 #ifdef	__cplusplus
 extern "C" {
@@ -45,6 +45,8 @@ typedef struct {
   unsigned char *buf;
   const unsigned char *orig;
   const unsigned char *next;
+  unsigned char *token;
+  uint32_t tlen;
   sen_nstr *nstr;
 #ifndef NO_MECAB
   mecab_t *mecab;
@@ -80,4 +82,4 @@ sen_rc sen_lex_validate(sen_sym *sym);
 }
 #endif
 
-#endif /* _SEN_LEX_H */
+#endif /* SEN_LEX_H */
