@@ -29,7 +29,9 @@
 #include "ctx.h"
 #endif /* SEN_CTX_H */
 
+#ifndef __USE_ISOC99
 #define __USE_ISOC99
+#endif /*  __USE_ISOC99 */
 #include <math.h>
 
 #ifdef __cplusplus
@@ -44,6 +46,7 @@ extern "C" {
 #define sen_ql_bulk          0x13
 #define sen_ql_int           0x14
 #define sen_ql_time          0x15
+#define sen_ql_symsnip       0x16
 #define sen_ql_float         0x17
 #define sen_ql_snip          0x18
 #define sen_ql_query         0x19
@@ -53,7 +56,8 @@ extern "C" {
 #define sen_ql_promise       0x1d
 #define sen_ql_closure       0x1e
 #define sen_ql_continuation  0x1f
-#define sen_ql_index         0x30
+#define sen_ql_macro         0x20
+#define sen_ql_index         0x30  /* only used in inv.c */
 #define sen_ql_list          0x40
 
 #define SEN_QL_CO_BEGIN(c) if (c) { switch((c)->last) { case 0:
@@ -95,11 +99,12 @@ extern sen_obj *sen_ql_f;    /* special cell representing #f */
 #define SEN_OBJ_ALLOCATED 1
 #define SEN_OBJ_MARKED 2
 #define SEN_OBJ_SYMBOL 4
-#define SEN_OBJ_MACRO  8
+#define SEN_OBJ_MARK2  8
 #define SEN_OBJ_PROMISE 16
 #define SEN_OBJ_REFERER 32
 #define SEN_OBJ_NATIVE 64
 #define SEN_OBJ_FROMJA 128
+#define SEN_OBJ_FREE 256
 
 void sen_ql_init_globals(sen_ctx *c);
 
@@ -129,6 +134,12 @@ sen_obj *sen_obj_cons(sen_ctx *ctx, sen_obj *a, sen_obj *b);
 char *sen_obj_copy_bulk_value(sen_ctx *ctx, sen_obj *o);
 void sen_ql_init_const(void);
 
+sen_rc sen_ql_obj_mark(sen_ctx *ctx, sen_obj *o);
+sen_rc sen_ql_obj_unmark(sen_ctx *ctx, sen_obj *o);
+
+typedef struct _symsnip_spec symsnip_spec;
+void sen_ql_symsnip_spec_close(sen_ctx *ctx, symsnip_spec *ss);
+
 #define SEN_OBJ2VALUE(o,v,s) ((v) = (o)->u.b.value, (s) = (o)->u.b.size)
 #define SEN_VALUE2OBJ(o,v,s) ((o)->u.b.value = (v), (o)->u.b.size = (s))
 
@@ -151,10 +162,15 @@ void sen_ql_init_const(void);
 #define SLOTP(c) (sen_db_obj_slot <= (c)->type && (c)->type <= sen_db_pslot)
 #define OPP(c) ((c)->type == sen_ql_op)
 #define STRVALUE(c) ((c)->u.b.value)
+#define STRSIZE(c) ((c)->u.b.size)
 #define NUMBERP(c) ((c)->type == sen_ql_int || (c)->type == sen_ql_float)
 #define SYMBOLP(c) ((c)->flags & SEN_OBJ_SYMBOL)
 #define SYMNAME(c) (SEN_SET_STRKEY_BY_VAL(c))
 #define KEYWORDP(c) (*SYMNAME(c) == ':')
+#define CONTINUATIONP(p) ((p)->type == sen_ql_continuation)
+#define CLOSUREP(p) ((p)->type == sen_ql_closure || (p)->type == sen_ql_macro)
+#define PROCP(p) ((p)->type == sen_ql_proc)
+#define PROCEDUREP(p) (PROCP(p) || CLOSUREP(p) || CONTINUATIONP(p) || NATIVE_FUNCP(p))
 
 #define SETINT(c,v) ((c)->type = sen_ql_int, (c)->u.i.i = (v))
 #define SETFLOAT(c,v) ((c)->type = sen_ql_float, (c)->u.d.d = (v))
@@ -191,6 +207,8 @@ void sen_ql_init_const(void);
 
 #define CONS(a,b) (sen_obj_cons(ctx, a, b))
 
+#define INTERN(s) (sen_ql_mk_symbol(ctx, s))
+
 #define POP(x,c) (PAIRP(c) ? ((x) = CAR(c), (c) = CDR(c), (x)) : (x = NIL))
 
 #define SKIPSPACE(c) do {\
