@@ -22,66 +22,6 @@
 #include "ctx.h"
 #include <string.h>
 
-/* simple array */
-
-void
-sen_array_init(sen_array *a, uint16_t element_size, uint16_t flags)
-{
-  a->element_size = element_size;
-  a->flags = flags;
-  MUTEX_INIT(a->lock);
-  memset(a->elements, 0, sizeof(a->elements));
-}
-
-void
-sen_array_fin(sen_array *a)
-{
-  int i;
-  for (i = 0; i < 8; i++) {
-    void **e = &a->elements[i];
-    if (*e) {
-      SEN_GFREE(*e);
-      *e = NULL;
-    }
-  }
-}
-
-void *
-sen_array_at(sen_array *a, sen_id id)
-{
-  int l, i, o;
-  byte **e;
-  SEN_BIT_SCAN_REV(id, l);
-  i = l >> 2;
-  o = 1 << (i << 2);
-  e = (byte **)&a->elements[i];
-  if (!*e) {
-    MUTEX_LOCK(a->lock);
-    if (!*e) {
-      if (a->flags & sen_array_clear) {
-        *e = SEN_GCALLOC(0x0f * o * a->element_size);
-      } else {
-        *e = SEN_GMALLOC(0x0f * o * a->element_size);
-      }
-    }
-    MUTEX_UNLOCK(a->lock);
-    if (!*e) { return NULL; }
-  }
-  return *e + (id - o) * a->element_size;
-}
-
-#define SEN_ARRAY_EACH(a,k,v,block) do {\
-  int _ei;\
-  for (_ei = 0, (k) = 1; _ei < 8; _ei++) {\
-    int _ej = 0x0f * (1 << (_ei << 2));\
-    if (((v) = (a)->elements[_ei])) {\
-      for (; _ej--; (k)++, (v) = (void *)((byte *)(v) + (a)->element_size)) block\
-    } else {\
-      (k) += _ej;\
-    }\
-  }\
-} while (0)
-
 /* rectangular arrays */
 
 #define SEN_RA_IDSTR "SENNA:RA:01.000"
@@ -630,7 +570,7 @@ struct sen_ja_header {
 
 // segment_width, align_width, flags
 sen_ja *
-sen_ja_create(const char *path, unsigned int max_element_size)
+sen_ja_create(const char *path, unsigned int max_element_size, uint32_t flags)
 {
   int i;
   sen_io *io;
@@ -652,7 +592,7 @@ sen_ja_create(const char *path, unsigned int max_element_size)
   if (!io) { return NULL; }
   header = sen_io_header(io);
   memcpy(header->idstr, SEN_JA_IDSTR, 16);
-  header->flags = 0;
+  header->flags = flags;
   header->align_width = align_width;
   header->segment_width = segment_width;
   header->element_size = max_element_size;
@@ -719,8 +659,8 @@ sen_ja_remove(const char *path)
   return sen_io_remove(path);
 }
 
-void *
-sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len)
+inline static void *
+sen_ja_ref_raw(sen_ja *ja, sen_id id, uint32_t *value_len)
 {
   sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
   sen_io_ja_einfo *ep;
@@ -744,6 +684,105 @@ sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len)
   }
 }
 
+#ifndef NO_ZLIB
+#include <zlib.h>
+
+inline static void *
+sen_ja_ref_zlib(sen_ja *ja, sen_id id, uint32_t *value_len)
+{
+  sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+  z_stream zstream;
+  void *value, *zvalue;
+  uint32_t zvalue_len = *value_len + sizeof (uint64_t);
+  if (!(zvalue = sen_ja_ref_raw(ja, id, &zvalue_len))) { *value_len = 0; return NULL; }
+  zstream.next_in = (Bytef *)((uint64_t *)zvalue + 1);
+  zstream.avail_in = zvalue_len - sizeof (uint64_t);
+  zstream.zalloc = Z_NULL;
+  zstream.zfree = Z_NULL;
+  if (inflateInit2(&zstream, 15 /* windowBits */) != Z_OK) {
+    SEN_FREE((sen_io_ja_ehead *)zvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  if (!(value = SEN_MALLOC(*(uint64_t *)zvalue + sizeof (sen_io_ja_ehead)))) {
+    inflateEnd(&zstream);
+    SEN_FREE((sen_io_ja_ehead *)zvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  zstream.next_out = (Bytef *)((sen_io_ja_ehead *)value + 1);
+  zstream.avail_out = *(uint64_t *)zvalue;
+  if (inflate(&zstream, Z_FINISH) != Z_STREAM_END) {
+    inflateEnd(&zstream);
+    SEN_FREE(value);
+    SEN_FREE((sen_io_ja_ehead *)zvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  *value_len = zstream.total_out;
+  if (inflateEnd(&zstream) != Z_OK) {
+    SEN_FREE(value);
+    SEN_FREE((sen_io_ja_ehead *)zvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  *(sen_io_ja_ehead *)value = ((sen_io_ja_ehead *)zvalue)[-1];
+  SEN_FREE((sen_io_ja_ehead *)zvalue - 1);
+  return (sen_io_ja_ehead *)value + 1;
+}
+#endif /* NO_ZLIB */
+
+#ifndef NO_LZO
+#include <lzo/lzo1x.h>
+
+inline static void *
+sen_ja_ref_lzo(sen_ja *ja, sen_id id, uint32_t *value_len)
+{
+  sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+  void *value, *lvalue;
+  uint32_t lvalue_len = *value_len + sizeof (uint64_t);
+  lzo_uint loutlen;
+  if (!(lvalue = sen_ja_ref_raw(ja, id, &lvalue_len))) { *value_len = 0; return NULL; }
+  if (!(value = SEN_MALLOC(*(uint64_t *)lvalue + sizeof (sen_io_ja_ehead)))) {
+    SEN_FREE((sen_io_ja_ehead *)lvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  loutlen = *(uint64_t *)lvalue;
+  switch (lzo1x_decompress((lzo_bytep)((uint64_t *)lvalue + 1), lvalue_len - sizeof (uint64_t),
+                           (lzo_bytep)((sen_io_ja_ehead *)value + 1), &loutlen, NULL)) {
+  case LZO_E_OK :
+  case LZO_E_INPUT_NOT_CONSUMED :
+    break;
+  default :
+    SEN_FREE(value);
+    SEN_FREE((sen_io_ja_ehead *)lvalue - 1);
+    *value_len = 0;
+    return NULL;
+  }
+  *value_len = loutlen;
+  *(sen_io_ja_ehead *)value = ((sen_io_ja_ehead *)lvalue)[-1];
+  SEN_FREE((sen_io_ja_ehead *)lvalue - 1);
+  return (sen_io_ja_ehead *)value + 1;
+}
+#endif /* NO_LZO */
+
+void *
+sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len)
+{
+#ifndef NO_ZLIB
+  if (ja->header->flags & SEN_JA_ZLIB) {
+    return sen_ja_ref_zlib(ja, id, value_len);
+  }
+#endif /* NO_ZLIB */
+#ifndef NO_LZO
+  if (ja->header->flags & SEN_JA_LZO) {
+    return sen_ja_ref_lzo(ja, id, value_len);
+  }
+#endif /* NO_LZO */
+  return sen_ja_ref_raw(ja, id, value_len);
+}
+
 sen_rc
 sen_ja_unref(sen_ja *ja, sen_id id, void *value, uint32_t value_len)
 {
@@ -802,8 +841,8 @@ assign_rec(sen_ja *ja, int value_len)
   }
 }
 
-sen_rc
-sen_ja_put(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
+inline static sen_rc
+sen_ja_put_raw(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
 {
   sen_rc rc = sen_success;
   sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
@@ -867,6 +906,78 @@ sen_ja_put(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
   return rc;
 }
 
+#ifndef NO_ZLIB
+inline static sen_rc
+sen_ja_put_zlib(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
+{
+  sen_rc rc;
+  sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+  z_stream zstream;
+  void *zvalue;
+  int zvalue_len;
+  zstream.next_in = value;
+  zstream.avail_in = value_len;
+  zstream.zalloc = Z_NULL;
+  zstream.zfree = Z_NULL;
+  if (deflateInit2(&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 15 /* windowBits */,
+                   8 /* memLevel */, Z_DEFAULT_STRATEGY) != Z_OK) { return sen_other_error; }
+  zvalue_len = deflateBound(&zstream, value_len);
+  if (!(zvalue = SEN_MALLOC(zvalue_len + sizeof (uint64_t)))) { deflateEnd(&zstream); return sen_memory_exhausted; }
+  zstream.next_out = (Bytef *)((uint64_t *)zvalue + 1);
+  zstream.avail_out = zvalue_len;
+  if (deflate(&zstream, Z_FINISH) != Z_STREAM_END) {
+    deflateEnd(&zstream);
+    SEN_FREE(zvalue);
+    return sen_other_error;
+  }
+  zvalue_len = zstream.total_out;
+  if (deflateEnd(&zstream) != Z_OK) { SEN_FREE(zvalue); return sen_other_error; }
+  *(uint64_t *)zvalue = value_len;
+  rc = sen_ja_put_raw(ja, id, zvalue, zvalue_len + sizeof (uint64_t), flags);
+  SEN_FREE(zvalue);
+  return rc;
+}
+#endif /* NO_ZLIB */
+
+#ifndef NO_LZO
+inline static sen_rc
+sen_ja_put_lzo(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
+{
+  sen_rc rc;
+  sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+  void *lvalue, *lwork;
+  lzo_uint lvalue_len = value_len + value_len / 16 + 64 + 3;
+  if (!(lvalue = SEN_MALLOC(lvalue_len + sizeof (uint64_t)))) { return sen_memory_exhausted; }
+  if (!(lwork = SEN_MALLOC(LZO1X_1_MEM_COMPRESS))) { SEN_FREE(lvalue); return sen_memory_exhausted; }
+  if (lzo1x_1_compress(value, value_len, (lzo_bytep)((uint64_t *)lvalue + 1), &lvalue_len, lwork) != LZO_E_OK) {
+    SEN_FREE(lwork);
+    SEN_FREE(lvalue);
+    return sen_other_error;
+  }
+  SEN_FREE(lwork);
+  *(uint64_t *)lvalue = value_len;
+  rc = sen_ja_put_raw(ja, id, lvalue, lvalue_len + sizeof (uint64_t), flags);
+  SEN_FREE(lvalue);
+  return rc;
+}
+#endif /* NO_LZO */
+
+sen_rc
+sen_ja_put(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
+{
+#ifndef NO_ZLIB
+  if (ja->header->flags & SEN_JA_ZLIB) {
+    return sen_ja_put_zlib(ja, id, value, value_len, flags);
+  }
+#endif /* NO_ZLIB */
+#ifndef NO_LZO
+  if (ja->header->flags & SEN_JA_LZO) {
+    return sen_ja_put_lzo(ja, id, value, value_len, flags);
+  }
+#endif /* NO_LZO */
+  return sen_ja_put_raw(ja, id, value, value_len, flags);
+}
+
 static sen_rc
 sen_ja_defrag_seg(sen_ja *ja, uint32_t dseg)
 {
@@ -999,17 +1110,17 @@ sen_db_slot_class(sen_db *s, const char *slot)
 sen_rc
 sen_db_idx_slot_build(sen_db *s, sen_db_store *store)
 {
-  sen_index *index = store->u.i.index;
+  sen_inv *inv = store->u.i.inv;
+  sen_db_store *k = sen_db_store_by_id(s, store->u.i.class); // todo: support surrogate class
   sen_id id = SEN_SYM_NIL; /* maxid = sen_sym_curr_id(index->keys); */
   sen_db_store *target = sen_db_store_by_id(s, store->triggers->target);
-  if (!target) { return sen_invalid_argument; }
-  while ((id = sen_sym_next(index->keys, id))) {
+  if (!k || !target) { return sen_invalid_argument; }
+  while ((id = sen_sym_next(k->u.c.keys, id))) {
     uint32_t vs;
-    const char *key = _sen_sym_key(index->keys, id);
     void *vp = (void *)sen_ja_ref(target->u.v.ja, id, &vs);
     if (vp) {
-      if (key && vs && sen_index_upd(index, key, NULL, 0, vp, vs)) {
-        SEN_LOG(sen_log_error, "sen_index_upd failed. id=%d key=(%s)", id, key);
+      if (vs && sen_inv_upd(inv, id, NULL, NULL, 0, vp, vs)) {
+        SEN_LOG(sen_log_error, "sen_inv_upd failed. id=%d", id);
       }
       sen_ja_unref(target->u.v.ja, id, vp, vs);
     }
@@ -1067,10 +1178,7 @@ sen_db_store_open(sen_db *s, const char *name)
     break;
   case sen_db_idx_slot :
     e->u.i.class = spec->u.s.class;
-    if (!(e->u.i.index =
-          sen_index_open_with_keys_lexicon(buffer, k->u.c.keys, l->u.c.keys))) {
-      goto exit;
-    }
+    if (!(e->u.i.inv = sen_inv_open(buffer, l->u.c.keys))) { goto exit; }
     break;
   case sen_db_rel1 :
     e->u.f.class = spec->u.s.class;
@@ -1173,15 +1281,11 @@ sen_db_store_create(sen_db *s, const char *name, sen_db_store_spec *spec)
     break;
   case sen_db_ja_slot :
     e->u.v.class = spec->u.s.class;
-    if (!(e->u.v.ja = sen_ja_create(buffer, spec->u.s.size))) { goto exit; }
+    if (!(e->u.v.ja = sen_ja_create(buffer, spec->u.s.size, spec->u.s.flags))) { goto exit; }
     break;
   case sen_db_idx_slot :
     e->u.i.class = spec->u.s.class;
-    if (!(e->u.i.index =
-          sen_index_create_with_keys_lexicon(buffer, k->u.c.keys, l->u.c.keys,
-                                             spec->u.s.size))) {
-      goto exit;
-    }
+    if (!(e->u.i.inv = sen_inv_create(buffer, l->u.c.keys, spec->u.s.size))) { goto exit; }
     break;
   case sen_db_rel1 :
     e->u.f.class = spec->u.s.class;
@@ -1357,7 +1461,7 @@ sen_db_store_close(sen_db_store *slot)
     sen_ja_close(slot->u.v.ja);
     break;
   case sen_db_idx_slot :
-    sen_index_close(slot->u.i.index);
+    sen_inv_close(slot->u.i.inv);
     break;
   case sen_db_class :
     sen_sym_close(slot->u.c.keys);
@@ -1493,11 +1597,12 @@ sen_db_create(const char *path, int flags, sen_encoding encoding)
   ERRCLR(NULL);
   if (strlen(path) <= PATH_MAX - 14) {
     if ((s = SEN_GMALLOC(sizeof(sen_db)))) {
-      sen_array_init(&s->stores, sizeof(sen_db_store), sen_array_clear);
+      sen_array_init(&s->stores, &sen_gctx, sizeof(sen_db_store),
+                     SEN_ARRAY_CLEAR|SEN_ARRAY_THREADSAFE);
       if ((s->keys = sen_sym_create(path, 0, flags, encoding))) {
         char buffer[PATH_MAX];
         gen_pathname(path, buffer, 0);
-        if ((s->values = sen_ja_create(buffer, JA_SEGMENT_SIZE))) {
+        if ((s->values = sen_ja_create(buffer, JA_SEGMENT_SIZE, 0))) {
           MUTEX_INIT(s->lock);
           sen_db_prepare_builtin_class(s);
           if (!ERRP(&sen_gctx, SEN_ERROR)) {
@@ -1531,7 +1636,8 @@ sen_db_open(const char *path)
   ERRCLR(NULL);
   if (strlen(path) <= PATH_MAX - 14) {
     if ((s = SEN_GMALLOC(sizeof(sen_db)))) {
-      sen_array_init(&s->stores, sizeof(sen_db_store), sen_array_clear);
+      sen_array_init(&s->stores, &sen_gctx, sizeof(sen_db_store),
+                     SEN_ARRAY_CLEAR|SEN_ARRAY_THREADSAFE);
       if ((s->keys = sen_sym_open(path))) {
         char buffer[PATH_MAX];
         gen_pathname(path, buffer, 0);
@@ -1563,16 +1669,16 @@ sen_rc
 sen_db_close(sen_db *s)
 {
   if (!s) { return sen_invalid_argument; }
-  sen_sym_close(s->keys);
-  sen_ja_close(s->values);
   {
     sen_id id;
     sen_db_store *sp;
-    SEN_ARRAY_EACH(&s->stores, id, sp, {
+    SEN_ARRAY_EACH(&s->stores, 1, sen_sym_curr_id(s->keys), id, sp, {
       if (sp->type) { sen_db_store_close(sp); }
     });
   }
   sen_array_fin(&s->stores);
+  sen_sym_close(s->keys);
+  sen_ja_close(s->values);
   SEN_GFREE(s);
   return sen_success;
 }
@@ -1750,7 +1856,6 @@ sen_vgram_update(sen_vgram *vgram, sen_id rid, sen_vgram_buf *b, sen_set *terms)
         arg.mode = sen_sort_descending;
         arg.compar = NULL;
         arg.compar_arg = (void *)(intptr_t)sizeof(sen_id);
-        arg.compar_arg0 = NULL;
         ehs = sen_set_sort(th, 0, &arg);
         if (!ehs) {
           SEN_GFREE(ps);
