@@ -488,15 +488,15 @@ normalize_utf8(sen_nstr *nstr)
   int16_t *ch;
   sen_ctx *ctx = nstr->ctx;
   const unsigned char *s, *s_, *s__, *p, *p2, *pe, *e;
-  unsigned char *d, *d_;
-  uint_least8_t *cp, *ctypes;
-  size_t length = 0, ls, lp, size = nstr->orig_blen;
+  unsigned char *d, *d_, *de;
+  uint_least8_t *cp;
+  size_t length = 0, ls, lp, size = nstr->orig_blen, ds = size * 3;
   int removeblankp = nstr->flags & SEN_STR_REMOVEBLANK;
-  if (!(nstr->norm = SEN_MALLOC(size * 5 + 1))) { /* todo: realloc unless enough */
+  if (!(nstr->norm = SEN_MALLOC(ds + 1))) {
     return sen_memory_exhausted;
   }
   if (nstr->flags & SEN_STR_WITH_CHECKS) {
-    if (!(nstr->checks = SEN_MALLOC(size * 5 * sizeof(int16_t) + 1))) { /* todo: realloc unless enough */
+    if (!(nstr->checks = SEN_MALLOC(ds * sizeof(int16_t) + 1))) {
       SEN_FREE(nstr->norm);
       nstr->norm = NULL;
       return sen_memory_exhausted;
@@ -504,18 +504,20 @@ normalize_utf8(sen_nstr *nstr)
   }
   ch = nstr->checks;
   if (nstr->flags & SEN_STR_WITH_CTYPES) {
-    if (!(nstr->ctypes = SEN_MALLOC(size * 3 + 1))) { /* todo: realloc unless enough */
-      SEN_FREE(nstr->checks);
-      SEN_FREE(nstr->norm);
-      nstr->checks = NULL;
-      nstr->norm = NULL;
+    if (!(nstr->ctypes = SEN_MALLOC(ds + 1))) {
+      if (nstr->checks) {
+        SEN_FREE(nstr->checks); nstr->checks = NULL;
+      }
+      SEN_FREE(nstr->norm); nstr->norm = NULL;
       return sen_memory_exhausted;
     }
   }
-  cp = ctypes = nstr->ctypes;
+  cp = nstr->ctypes;
+  d = (unsigned char *)nstr->norm;
+  de = d + ds;
+  d_ = NULL;
   e = (unsigned char *)nstr->orig + size;
-  for (s = s_ = (unsigned char *)nstr->orig,
-       d = (unsigned char *)nstr->norm, d_ = NULL; ; s += ls) {
+  for (s = s_ = (unsigned char *)nstr->orig; ; s += ls) {
     if (!(ls = sen_str_charlen_utf8(s, e))) {
       break;
     }
@@ -540,10 +542,45 @@ normalize_utf8(sen_nstr *nstr)
       if (!(lp = sen_str_charlen_utf8(p, pe))) {
         break;
       }
-      if ((*p == ' ' && removeblankp)
-          || *p < 0x20  /* skip unprintable ascii */ ) {
-        if (cp > ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
+      if ((*p == ' ' && removeblankp) || *p < 0x20  /* skip unprintable ascii */ ) {
+        if (cp > nstr->ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
       } else {
+        if (de <= d + lp) {
+          unsigned char *norm;
+          ds += (ds >> 1) + lp;
+          if (!(norm = SEN_REALLOC(nstr->norm, ds + 1))) {
+            if (nstr->ctypes) { SEN_FREE(nstr->ctypes); nstr->ctypes = NULL; }
+            if (nstr->checks) { SEN_FREE(nstr->checks); nstr->checks = NULL; }
+            SEN_FREE(nstr->norm); nstr->norm = NULL;
+            return sen_memory_exhausted;
+          }
+          de = norm + ds;
+          d = norm + (d - (unsigned char *)nstr->norm);
+          nstr->norm = norm;
+          if (ch) {
+            int16_t *checks;
+            if (!(checks = SEN_REALLOC(nstr->checks, ds * sizeof(int16_t)+ 1))) {
+              if (nstr->ctypes) { SEN_FREE(nstr->ctypes); nstr->ctypes = NULL; }
+              SEN_FREE(nstr->checks); nstr->checks = NULL;
+              SEN_FREE(nstr->norm); nstr->norm = NULL;
+              return sen_memory_exhausted;
+            }
+            ch = checks + (ch - nstr->checks);
+            nstr->checks = checks;
+          }
+          if (cp) {
+            uint_least8_t *ctypes;
+            if (!(ctypes = SEN_REALLOC(nstr->ctypes, ds + 1))) {
+              SEN_FREE(nstr->ctypes); nstr->ctypes = NULL;
+              if (nstr->checks) { SEN_FREE(nstr->checks); nstr->checks = NULL; }
+              SEN_FREE(nstr->norm); nstr->norm = NULL;
+              return sen_memory_exhausted;
+            }
+            cp = ctypes + (cp - nstr->ctypes);
+            nstr->ctypes = ctypes;
+          }
+        }
+
         memcpy(d, p, lp);
         d_ = d;
         d += lp;
