@@ -517,7 +517,7 @@ sen_sym_create08(const char *path, uint32_t key_size,
   for (i = 0; i < SEN_SYM_MAX_SEGMENT; i++) {
     header->segments[i] = segment_empty;
   }
-  if (!(sym = SEN_MALLOC(sizeof(sen_sym08)))) {
+  if (!(sym = SEN_GMALLOC(sizeof(sen_sym08)))) {
     sen_io_close(io);
     return NULL;
   }
@@ -560,7 +560,7 @@ sen_sym_open08(const char *path)
     return NULL;
   }
   */
-  if (!(sym = SEN_MALLOC(sizeof(sen_sym08)))) {
+  if (!(sym = SEN_GMALLOC(sizeof(sen_sym08)))) {
     puts("malloc failed");
     sen_io_close(io);
     return NULL;
