@@ -148,7 +148,7 @@ take_sjis_to_utf8(const gchar *sjis)
 } while (0)
 
 void
-startup(void)
+cut_startup(void)
 {
   GError *error = NULL;
 
@@ -163,7 +163,7 @@ startup(void)
 }
 
 void
-shutdown(void)
+cut_shutdown(void)
 {
   if (text_ja_euc) {
     g_free(text_ja_euc);
@@ -175,7 +175,7 @@ shutdown(void)
 }
 
 void
-setup(void)
+cut_setup(void)
 {
   snip = NULL;
   keyword = NULL;
